package kaysaar.aotd_question_of_loyalty.data.plugins;

import com.fs.starfarer.api.Global;
import java.util.Set;
import com.fs.starfarer.api.campaign.econ.Industry;
import exerelin.campaign.ColonyManager;
import java.util.HashSet;
import com.fs.starfarer.api.impl.campaign.procgen.StarSystemGenerator;
import com.fs.starfarer.api.campaign.PersonImportance;
import com.fs.starfarer.api.impl.campaign.ids.Ranks;
import com.fs.starfarer.api.characters.ImportantPeopleAPI;
import java.util.Iterator;
import com.fs.starfarer.api.campaign.CommDirectoryEntryAPI;
import com.fs.starfarer.api.characters.PersonAPI;
import com.fs.starfarer.api.util.Misc;
import com.fs.starfarer.api.campaign.econ.MarketAPI;

public class AoTDNexMarketUtil
{
    public static boolean hasPerson(final MarketAPI market, final String postId) {
        return getPerson(market, postId) != null;
    }
    
    public static boolean doesPlayerHaveMarkets(final boolean includeNonPlayerFactionMarkets) {
        return Misc.getPlayerMarkets(includeNonPlayerFactionMarkets).stream().anyMatch(x -> !x.hasTag("nex_playerOutpost"));
    }
    
    public static PersonAPI getPerson(final MarketAPI market, final String postId) {
        for (final CommDirectoryEntryAPI dir : market.getCommDirectory().getEntriesCopy()) {
            if (dir.getType() != CommDirectoryEntryAPI.EntryType.PERSON) {
                continue;
            }
            final PersonAPI person = (PersonAPI)dir.getEntryData();
            if (person.getPostId().equals(postId)) {
                return person;
            }
        }
        return null;
    }
    
    public static PersonAPI addPerson(final ImportantPeopleAPI ip, final MarketAPI market, final String rankId, final String postId, final boolean noDuplicate) {
        if (noDuplicate && hasPerson(market, postId)) {
            return null;
        }
        final PersonAPI person = market.getFaction().createRandomPerson();
        if (rankId != null) {
            person.setRankId(rankId);
        }
        person.setPostId(postId);
        market.getCommDirectory().addPerson(person);
        market.addPerson(person);
        ip.addPerson(person);
        ip.getData(person).getLocation().setMarket(market);
        ip.checkOutPerson(person, "permanent_staff");
        if (postId.equals(Ranks.POST_BASE_COMMANDER) || postId.equals(Ranks.POST_STATION_COMMANDER) || postId.equals(Ranks.POST_ADMINISTRATOR)) {
            if (market.getSize() >= 8) {
                person.setImportanceAndVoice(PersonImportance.VERY_HIGH, StarSystemGenerator.random);
            }
            else if (market.getSize() >= 6) {
                person.setImportanceAndVoice(PersonImportance.HIGH, StarSystemGenerator.random);
            }
            else {
                person.setImportanceAndVoice(PersonImportance.MEDIUM, StarSystemGenerator.random);
            }
        }
        else if (postId.equals(Ranks.POST_PORTMASTER)) {
            if (market.getSize() >= 8) {
                person.setImportanceAndVoice(PersonImportance.HIGH, StarSystemGenerator.random);
            }
            else if (market.getSize() >= 6) {
                person.setImportanceAndVoice(PersonImportance.MEDIUM, StarSystemGenerator.random);
            }
            else if (market.getSize() >= 4) {
                person.setImportanceAndVoice(PersonImportance.LOW, StarSystemGenerator.random);
            }
            else {
                person.setImportanceAndVoice(PersonImportance.VERY_LOW, StarSystemGenerator.random);
            }
        }
        else if (postId.equals(Ranks.POST_SUPPLY_OFFICER)) {
            if (market.getSize() >= 6) {
                person.setImportanceAndVoice(PersonImportance.MEDIUM, StarSystemGenerator.random);
            }
            else if (market.getSize() >= 4) {
                person.setImportanceAndVoice(PersonImportance.LOW, StarSystemGenerator.random);
            }
            else {
                person.setImportanceAndVoice(PersonImportance.VERY_LOW, StarSystemGenerator.random);
            }
        }
        return person;
    }
    
    public static void addOrUpdateOfficials(final MarketAPI market) {
        final Set<String> officialsPresent = new HashSet<String>();
        for (final CommDirectoryEntryAPI dir : market.getCommDirectory().getEntriesCopy()) {
            if (dir.getType() != CommDirectoryEntryAPI.EntryType.PERSON) {
                continue;
            }
            final PersonAPI person = (PersonAPI)dir.getEntryData();
            if (person.getFaction() != market.getFaction()) {
                continue;
            }
            if (!ColonyManager.NEEDED_OFFICIALS.contains(person.getPostId())) {
                continue;
            }
            officialsPresent.add(person.getPostId());
        }
        boolean havePerson = !officialsPresent.isEmpty();
        if (market.hasIndustry("militarybase") || market.hasIndustry("highcommand")) {
            String rankId = Ranks.GROUND_MAJOR;
            if (market.getSize() >= 6) {
                rankId = Ranks.GROUND_GENERAL;
            }
            else if (market.getSize() >= 4) {
                rankId = Ranks.GROUND_COLONEL;
            }
            addOrUpdateOfficial(market, rankId, Ranks.POST_BASE_COMMANDER, officialsPresent);
            havePerson = true;
        }
        boolean hasStation = false;
        for (final Industry curr : market.getIndustries()) {
            if (curr.getSpec().hasTag("station")) {
                hasStation = true;
                break;
            }
        }
        if (hasStation) {
            String rankId2 = Ranks.SPACE_COMMANDER;
            if (market.getSize() >= 6) {
                rankId2 = Ranks.SPACE_ADMIRAL;
            }
            else if (market.getSize() >= 4) {
                rankId2 = Ranks.SPACE_CAPTAIN;
            }
            addOrUpdateOfficial(market, rankId2, Ranks.POST_STATION_COMMANDER, officialsPresent);
            havePerson = true;
        }
        if (market.hasSpaceport()) {
            addOrUpdateOfficial(market, null, Ranks.POST_PORTMASTER, officialsPresent);
            havePerson = true;
        }
        if (havePerson) {
            addOrUpdateOfficial(market, Ranks.SPACE_COMMANDER, Ranks.POST_SUPPLY_OFFICER, officialsPresent);
            havePerson = true;
        }
        addOrUpdateOfficial(market, Ranks.CITIZEN, Ranks.POST_ADMINISTRATOR, officialsPresent);
        havePerson = true;
    }
    
    public static void addOrUpdateOfficial(final MarketAPI market, final String rankId, final String postId, final Set<String> postsPresent) {
        if (postsPresent.contains(postId) && rankId != null) {
            final PersonAPI person = getPerson(market, postId);
            if (person != null && !rankId.equals(person.getRankId())) {
                person.setRankId(rankId);
            }
            return;
        }
        addPerson(Global.getSector().getImportantPeople(), market, rankId, postId, true);
        postsPresent.add(postId);
    }
}
